<?php

namespace Drupal\lms\Entity\Handlers;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\lms\Entity\Bundle\Course;

/**
 * Access controller for the Answer entity.
 *
 * @see \Drupal\lms\Entity\Answer.
 */
class AnswerAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    if ($operation === 'evaluate') {
      // Check if the account is a member of the course this answer belongs to.
      $course = $entity->getLessonStatus()->getCourseStatus()->getCourse();
      if ($course instanceof Course) {
        $membership = $course->getMember($account);
        if ($membership !== FALSE) {
          return AccessResult::allowed()->addCacheableDependency($membership);
        }
      }
    }
    return AccessResult::allowedIfHasPermission($account, 'administer lms');
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    // We create answers only as a result of ansewer form logic.
    return AccessResult::forbidden();
  }

}
