<?php

namespace Drupal\lms\Entity\Handlers;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\lms\Entity\Lesson;

/**
 * Defines a class to build a listing of Lesson entities.
 */
class LessonListBuilder extends EntityListBuilder {

  /**
   * Returns lesson list.
   */
  protected function getEntities() {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->sort($this->entityType->getKey('id'));
    $ids = $query->execute();

    // Filter entities that user has edit access.
    $entities = Lesson::loadMultiple($ids);
    $entities = array_filter($entities, function ($entity) {
      return $entity->access('update');
    });
    return $entities;
  }

  /**
   * Returns lessons count.
   */
  protected function getTotalCount() {
    return count($this->getEntities());
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    $ids = array_keys($this->getEntities());

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $page = \Drupal::request()->query->get('page', 0);
      $limit = $this->limit;
      $start = $limit * $page;
      $ids = array_slice($ids, $start, $limit);
    }

    return $ids;
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $pager_manager = \Drupal::service('pager.manager');
    $pager_manager->createPager($this->getTotalCount(), $this->limit);

    $build = parent::render();
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('Lesson ID');
    $header['name'] = $this->t('Name');
    $header['owner'] = $this->t('Owner');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['id'] = $entity->id();
    $row['name'] = Link::createFromRoute($entity->label(), 'entity.lms_lesson.edit_form', ['lms_lesson' => $entity->id()]);
    $row['owner'] = $entity->get('uid')->entity->toLink();

    // Change default edit link.
    $ops = parent::buildRow($entity);
    $destination = $_SERVER['REQUEST_URI'];
    $ops['operations']['data']['#links']['edit']['url'] = new Url(
      'entity.lms_lesson.edit_form', [
        'lms_lesson' => $entity->id(),
      ], [
        'query' => ['destination' => $destination],
      ]
    );

    return $row + $ops;
  }

}
